#ifndef _RINGBUF_H_
#define _RINGBUF_H_

#define MIN(A, B) ((A)<(B)?(A):(B))

typedef struct {
    char* buffer; 
    int read;
    int write;
    int size;
} ringbuff_t;

extern inline void rb_init(ringbuff_t *rb, char* _rb, int sze);
extern inline char* rb_putblock(ringbuff_t *rb, int* len);
extern inline void rb_markblock(ringbuff_t *rb, int len);
extern inline char* rb_getblock(ringbuff_t *rb, int* len);
extern inline void rb_freeblock(ringbuff_t *rb, char* addr,int sze);
extern inline int rb_isempty(ringbuff_t *rb);
extern inline int rb_isfull(ringbuff_t *rb);

extern inline void rb_init(ringbuff_t *rb, char* _rb,int sze) {
    rb->buffer=_rb;
    rb->read=-1;
    rb->write=0;
    rb->size=sze;
}

extern inline char* rb_putblock(ringbuff_t *rb, int* len) {
    char *dst;
    if(-1==rb->write) {
	*len=0;
	dst=0;
    } else {
	if(rb->read<=rb->write) *len=MIN(*len, rb->size - rb->write);
	else                  *len=MIN(*len, rb->read - rb->write);
	dst=&(rb->buffer[rb->write]);
    }
    return dst;
}

extern inline void rb_markblock(ringbuff_t *rb, int len) {
    if(0==len||-1==rb->write) return;
    if(-1==rb->read) rb->read=rb->write;
    rb->write=(rb->write+len)%rb->size;
    if (rb->write==rb->read) rb->write=-1;
}

extern inline char* rb_getblock(ringbuff_t *rb, int* len) {
    if(-1==rb->read) {
	*len=0;
	return 0;
    }
    if(rb->write<=rb->read) *len=MIN(*len, rb->size - rb->read);
    else *len=MIN(*len, rb->write - rb->read);
    return &(rb->buffer[rb->read]);
}

extern inline void rb_freeblock(ringbuff_t *rb, char* addr,int sze) {
    if(-1==rb->write) rb->write=rb->read;
    rb->read=(rb->read+sze)%rb->size;
    if(rb->read==rb->write) rb->read=-1;
}

extern inline int rb_isempty(ringbuff_t *rb) {
    return (-1 == rb->read);
}

extern inline int rb_isfull(ringbuff_t *rb) {
    return (-1 == rb->write);
}

#undef MIN

#endif
